# -*- coding: utf-8 -*-
from __future__ import absolute_import

from pkg_resources import get_distribution

from pysaint.api import get
from pysaint.utils import save_json

__version__ = get_distribution('pysaint').version

__all__ = [
    'get',
    'save_json',
]



"""
    End User를 위한 간단한 api


"""

from .saint import Saint
import copy


def get(course_type, year_range, semesters, **kwargs):
    """
    THIS IS THE END POINT OF pysaint API
    USAGE::

        >>> import pysaint
        >>> res = pysaint.get('전공', ['2018'], ['2 학기'])
        >>> print(res)


        >>> res = pysaint.get('교양필수', range(2015, 2017), ['1 학기', '여름학기', '2 학기', '겨울학기'])
        >>> print(res)


        >>> res = pysaint.get('교양선택', (2016, 2017, 2018), ('1 학기', ))
        >>> print(res)

    TODO: add log option to decide silent or not

    :param course_type:
    :type course_type: str
    example )
            '교양필수'
            '전공'
            '교양선택'
    :param year_range:
    :type year_range: list or tuple or range
    :param semesters:
    :type semesters: list or tuple
    :return: dict
    """

    if type(year_range) not in (tuple, list, range):
        raise ValueError("get() got wrong arguments year_range: {}\n"
                         "expected tuple type or list, or range type but got {} type".format(year_range, type(year_range)))

    if type(semesters) not in (tuple, list):
        raise ValueError("get() got wrong arguments semesters: {}\n"
                         "expected tuple type or list type but got {} type".format(semesters, type(semesters)))

    reformed_year_range = []
    for year in year_range:
        if 2000 < int(year) < 2020:
            pass
        else:
            raise ValueError("get() got wrong arguments year_range: {}\n"
                             "expected to be in year range(2000, 2020) but got {}".format(year_range, int(year)))
        reformed_year_range.append("{}".format(year))

    if course_type == '교양필수':
        return _liberal_arts(year_range=reformed_year_range, semesters=semesters, **kwargs)
    elif course_type == '전공':
        return _major(year_range=reformed_year_range, semesters=semesters, **kwargs)
    elif course_type == '교양선택':
        return _selective_liberal(year_range=reformed_year_range, semesters=semesters, **kwargs)
    else:
        raise ValueError("get() got wrong arguments course_type: {} \n"
                         "expected to get '교양필수', '전공', '교양선택'".format(course_type))


def _liberal_arts(year_range=[], semesters=[], **kwargs):
    """
    교양필수 과목들을 학기 단위로 묶어서 반환한다.
    :param year_range:
    :type year_range: list or tuple
    example input )
            [2013, 2014, 2015, 2016, 2017, 2018]
            or
            (2017, 2018)
    :param semesters:
    :type semesters: list or tuple
    example input )
            ['1 학기', '여름학기', '2 학기', '겨울학기']
            or
            ('1 학기')
    :return:
    {
        2013: {
            '전체학년': {
                'CHAPEL': [
                    {
                        dictionary which has
                        dict_keys(['계획', '이수구분(주전공)',
                        '이수구분(다전공)', '공학인증', '교과영역',
                        '과목번호', '과목명', '분반', '교수명',
                        '개설학과', '시간/학점(설계)', '수강인원',
                        '여석', '강의시간(강의실)', '수강대상'])
                    }
                ],
                '컴퓨터활용1(Excel)': [],
                '컴퓨터활용2(PPT)': [],
                'Practical Reading ＆ Writing': [],
                '현대인과성서2': []
                }
            }
            '1학년': {...},
            '2학년': {...},
            '3학년': {...},
            '4학년': {...},
            '5학년': {...}
        },
        year: {
            grade: {
                course_name: [] <- list which has dictionaries as it's elements
            }
        }
    }
    """
    ret = {year: {} for year in year_range}
    saint = Saint()
    saint.select_course_section('교양필수')

    def __get_whole_course(year, semester):
        print('{} {}'.format(year, semester))
        saint.select_year(year)
        saint.select_semester(semester)
        liberal_map = saint.get_liberal_arts_map()
        course_map = copy.deepcopy(liberal_map)

        for grade in liberal_map:
            course_map[grade] = {course_name: {} for course_name in liberal_map[grade]}

        for grade in liberal_map:
            for course_name in liberal_map[grade]:
                if course_name != '':
                    course_map[grade][course_name] = saint.select_on_liberal_arts(grade, course_name)

        return course_map

    for year in year_range:
        for semester in semesters:
            course_bunch = __get_whole_course(year, semester)
            ret[year][semester] = course_bunch

    return ret


def _major(year_range=[], semesters=[], **kwargs):
    """
    전공 과목들을 학기 단위로 묶어서 반환한다.
    :param year_range:
    :type year_range: list or tuple
    :param semesters:
    :type semesters: list or tuple
    :return:
    {
        '2017': {
            '1 학기': {
                '인문대학': {
                    '중어중문학과': {
                        '중어중문학과': [
                            {
                                '계획': '\xa0',
                                '이수구분(주전공)': '전선-중문',
                                '이수구분(다전공)': '복선-중문/부선-중문',
                                '공학인증': '\xa0',
                                '교과영역': '7+1교과목\n인턴쉽(장기과정)\n인턴쉽',
                                '과목번호': '5010611601',
                                '과목명': '국내장기현장실습(3)',
                                '분반': '\xa0',
                                '교수명': '\xa0',
                                '개설학과': '경력개발팀',
                                '시간/학점(설계)': '3.00 /3',
                                '수강인원': '1',
                                '여석': '199',
                                '강의시간(강의실)': '\xa0',
                                '수강대상': '전체'
                            },
                            {
                                ...
                                dict_keys(['계획', '이수구분(주전공)', '이수구분(다전공)', '공학인증', '교과영역',
                                '과목번호', '과목명', '분반', '교수명', '개설학과', '시간/학점(설계)', '수강인원',
                                '여석', '강의시간(강의실)', '수강대상'])
                            }
                        ]
                    },
                    '국어국문학과': {},
                    '일어일문학과': {},
                    '영어영문학과': {},
                    '불어불문학과': {},
                    '철학과': {},
                    '사학과': {},
                    '기독교학과': {},
                },
                '자연과학대학': {},
                '법과대학': {},
                '사회과학대학': {},
                '경제통상대학': {},
                '경영대학': {},
                '공과대학': {},
                'IT대학': {},
                '베어드학부대학': {},
                '예술창작학부': {},
                '스포츠학부': {},
                '융합특성화자유전공학부': {}
            }
        },
        'year': {
            'semester': {
                'college': {
                    'faculty': {
                        'major': [
                            {
                                dict_keys(['계획', '이수구분(주전공)', '이수구분(다전공)', '공학인증', '교과영역',
                                '과목번호', '과목명', '분반', '교수명', '개설학과', '시간/학점(설계)', '수강인원',
                                '여석', '강의시간(강의실)', '수강대상'])
                            }
                        ]
                    }
                }
            }
        }
    }
    """

    ret = {year: {} for year in year_range}
    saint = Saint()

    def __get_whole_course(year, semester):
        print('{} {}'.format(year, semester))
        saint.select_year(year)
        saint.select_semester(semester)
        major_map = saint.get_major_map()
        course_map = copy.deepcopy(major_map)

        for college in major_map:
            for faculty in major_map[college]:
                course_map[college][faculty] = {key: [] for key in major_map[college][faculty]}

        for college in major_map:
            for faculty in major_map[college]:
                for major in major_map[college][faculty]:
                    print('{} {} {}'.format(college, faculty, major))
                    course_map[college][faculty][major] = saint.select_on_major(college, faculty, major)

        return course_map

    for year in year_range:
        for semester in semesters:
            course_bunch = __get_whole_course(year, semester)
            ret[year][semester] = course_bunch

    return ret


def _selective_liberal(year_range=[], semesters=[], **kwargs):
    """
    교양선택 과목들을 학기 단위로 묶어서 반환한다.
    :param year_range:
    :param semesters:
    :return: dict
    {
        2017: {
            '1 학기': {
                전체: [
                    {
                        '계획': '\xa0',
                        '이수구분(주전공)': '교선',
                        '이수구분(다전공)': '\xa0',
                        '공학인증': '\xa0',
                        '교과영역': '*세계의언어(핵심-창의)\n(기초역량-국제어문)영어',
                        '과목번호': '2150017601',
                        '과목명': 'Advanced Writing and speaking English I',
                        '분반': '\xa0',
                        '교수명': '이종일\n이종일\n이종일',
                        '개설학과': '벤처경영학과(계약학과)',
                        '시간/학점(설계)': '3.00 /3',
                        '수강인원': '11',
                        '여석': '39',
                        '강의시간(강의실)': '월 19:00-19:50 (조만식기념관 12530-이종일)\n월 20:00-20:50 (조만식기념관 12530-이종일)\n월 21:00-21:50 (조만식기념관 12530-이종일)',
                        '수강대상': '전체학년 벤처경영학과(계약학과) (대상외수강제한)(대상외수강제한)'
                    },
                    {
                        dict_keys(['계획', '이수구분(주전공)', '이수구분(다전공)',
                        '공학인증', '교과영역', '과목번호', '과목명', '분반', '교수명',
                        '개설학과', '시간/학점(설계)', '수강인원', '여석', '강의시간(강의실)', '수강대상'])
                    }
                ]
                *문학과 예술(융합-인문): []
                *역사와철학(융합-인문): []
                *정보와기술(융합-자연): []
                *창의성과의사소통능력(핵심-창의): []
                *세계의언어(핵심-창의): []
                *세계의문화와국제관계(핵심-창의): []
                *인간과사회(융합-사회): []
                *정치와경제(융합-사회): []
                *자연과학과수리(융합-자연): []
                *생활과건강(실용-생활): []
                *학문과진로탐색(실용-생활): []
                *인성과리더쉽(핵심-창의): []
                숭실품성(인성-종교가치인성교육): []
                숭실품성(인성-가치관및윤리교육): []
                숭실품성(인성-공동체인성교육): []
                숭실품성(리더십-통일리더십): []
                숭실품성(리더십-리더십이론및실천): []
                기초역량(사고력-논리및비판적사고): []
                기초역량(사고력-창의및융합적사고): []
                기초역량(사고력-수리적사고): []
                기초역량(한국어의사소통-읽기와쓰기): []
                기초역량(한국어의사소통-의사소통): []
                (기초역량-국제어문)영어: []
                기초역량(국제어문-국제어): []
                기초역량(국제어문-고전어문 ): []
                기초역량(과학정보기술-과학): []
                기초역량(과학정보기술-정보기술): []
                균형교양(인문학-문학/어학/예술): []
                균형교양(인문학-역사): []
                균형교양(인문학-철학/사상): []
                균형교양(사회과학-사회/정치/경제): []
                균형교양(사회과학-문화및문명): []
                균형교양(자연과학-자연과학): []
                실용교양(개인과가족생활): []
                실용교양(경제경영): []
                실용교양(공공생활): []
                실용교양(기술생활): []
                실용교양(자기개발과진로탐색)
            }
        },
        year: {
            'semester': {
                'section': [
                    {
                        dict_keys(['계획', '이수구분(주전공)', '이수구분(다전공)',
                        '공학인증', '교과영역', '과목번호', '과목명', '분반', '교수명',
                        '개설학과', '시간/학점(설계)', '수강인원', '여석', '강의시간(강의실)', '수강대상'])
                    }
                ]
            }
        }
    }
    """
    ret = {year: {} for year in year_range}
    saint = Saint()
    saint.select_course_section('교양선택')

    # is this necessary job?
    saint.select_year('2017')
    saint.select_semester('2 학기')

    def __get_whole_course(year, semester):
        print('{} {}'.format(year, semester))
        saint.select_year(year)
        saint.select_semester(semester)
        selective_map = saint.get_selective_liberal_map()
        course_map = {course_name: {} for course_name in selective_map}

        for course_name in selective_map:
            if course_name != '':
                course_map[course_name] = saint.select_on_selective_liberal(course_name)

        return course_map

    for year in year_range:
        for semester in semesters:
            course_bunch = __get_whole_course(year, semester)
            ret[year][semester] = course_bunch

    return ret


def _cyber(year_range=[], semesters=[], **kwargs):
    """
    TODO:
    시간나면 만들기
    :param year_range:
    :param semesters:
    :return:
    """



SOURCE_URL = "http://ecc.ssu.ac.kr/sap/bc/webdynpro/sap/zcmw2100?sap-language=KO#"
ECC_URL = "http://ecc.ssu.ac.kr"
REQUEST_HEADERS = {
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
    'Accept-Encoding': 'gzip, deflate, br',
    'Accept-Language': 'ko-KR,ko;q=0.9,en-US;q=0.8,en;q=0.7',
    'Cache-Control': 'max-age=0',
    'connection': 'keep-alive',
    'Content-Type': 'application/x-www-form-urlencoded',
    'DNT': '1',
    'Host': 'saint.ssu.ac.kr',
    'Origin': 'https://saint.ssu.ac.kr',
    'Referer': 'http://saint.ssu.ac.kr/irj/portal',
    'Upgrade-Insecure-Requests': '1',
    'user-agent': "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6)" +
                  "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.100 Safari/537.36",
}


SESSION_HEADERS = {
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
    'Accept-Encoding': 'gzip, deflate, br',
    'Accept-Language': 'ko-KR,ko;q=0.9,en-US;q=0.8,en;q=0.7',
    'Cache-Control': 'max-age=0',
    'connection': 'keep-alive',
    'Content-Type': 'application/x-www-form-urlencoded',
    'DNT': '1',
    'Upgrade-Insecure-Requests': '1',
    'user-agent': "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6)" +
                  "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.100 Safari/537.36",
}



"""
    make SAP Event data and return

"""


def combo_select(key_id, skey, sap_wd_secure_id):
    data = {
        'sap-charset': 'utf-8',
        'sap-wd-secure-id': sap_wd_secure_id,
        '_stateful_': 'X',
        'SAPEVENTQUEUE': 'ComboBox_Select~E002Id' +
                         '~E004{}~E005Key'.format(key_id) +
                         '~E004{skey}~E005ByEnter'.format(skey=skey) +  # ~E004xxx 에서 xxx에 sKey 대입
                         '~E004false~E003' +
                         '~E002ResponseData' +
                         '~E004delta' +
                         '~E005ClientAction' +
                         '~E004submit' +
                         '~E003~E002~E003',
    }

    return data


def button_press(button_id, sap_wd_secure_id):
    data = {
        'sap-charset': 'utf-8',
        'sap-wd-secure-id': sap_wd_secure_id,
        '_stateful_': 'X',
        'SAPEVENTQUEUE': 'Button_Press~E002Id' +
                         '~E004{}'.format(button_id) +
                         '~E003' +
                         '~E002ResponseData' +
                         '~E004delta' +
                         '~E005ClientAction' +
                         '~E004submit' +
                         '~E003' +
                         '~E002' +
                         '~E003'
    }

    return data


def combo_select_with_button_press(key_id, skey, button_id, sap_wd_secure_id):
    data = {
        'sap-charset': 'utf-8',
        'sap-wd-secure-id': sap_wd_secure_id,
        '_stateful_': 'X',
        'SAPEVENTQUEUE': 'ComboBox_Select' +
                         '~E002Id' +
                         '~E004{}'.format(key_id) +
                         '~E005Key' +
                         '~E004{}'.format(skey) +
                         '~E005ByEnter' '+'
                         '~E004false' +
                         '~E003' +
                         '~E002ResponseData' +
                         '~E004delta' +
                         '~E005EnqueueCardinality' +
                         '~E004single' +
                         '~E003' +
                         '~E002' +
                         '~E003' +
                         '~E001Button_Press' +
                         '~E002Id' +
                         '~E004{}'.format(button_id) +
                         '~E003' +
                         '~E002ResponseData' +
                         '~E004delta' +
                         '~E005ClientAction' +
                         '~E004submit' +
                         '~E003' +
                         '~E002' +
                         '~E003'
    }
    return data


def tab_select(section_id, tab_id, item_index, sap_wd_secure_id):
    """
    :param section_id:
    result of get_section_id() method
    :param tab_id:
    result of get_tab_id() method
    :param item_index:
    result of get_tab_item_index() method
    :param sap_wd_secure_id:
    Saint.sap_wd_secure_id
    :return:
    """
    data = {
        'sap-charset': 'utf-8',
        'sap-wd-secure-id': sap_wd_secure_id,
        '_stateful_': 'X',
        "SAPEVENTQUEUE": "TabStrip_TabSelect" +
                         "~E002Id" +
                         "~E004{}".format(section_id) +
                         "~E005ItemId" +
                         "~E004{}".format(tab_id) +
                         "~E005ItemIndex" +
                         "~E004{}".format(item_index) +
                         "~E005FirstVisibleItemIndex" +
                         "~E004{}".format(-1) +
                         "~E003" +
                         "~E002ResponseData" +
                         "~E004delta" +
                         "~E005ClientAction" +
                         "~E004submit" +
                         "~E003" +
                         "~E002" +
                         "~E003"
    }
    return data



# python ./setup.py build
# python ./setup.py install
from setuptools import setup, find_packages
import os


ROOT = os.path.abspath(os.path.dirname(__file__))
VERSION = '1.4.41'


def get_requirements(filename):
    return open(os.path.join(ROOT, filename)).read().splitlines()


setup(
    name='pysaint',
    packages=find_packages(),
    include_package_data=True,
    install_requires=get_requirements('requirements.txt'),
    tests_require=get_requirements('test-requirements.txt'),
    version=VERSION,
    description='Python module for saint.ssu.ac.kr',
    long_description=open(os.path.join(ROOT, 'README.md')).read(),
    long_description_content_type='text/markdown',
    author='gomjellie',
    author_email='gomjellie@gmail.com',
    url='https://github.com/gomjellie/pysaint',
    download_url='https://pypi.python.org/pypi/pysaint',
    license='MIT',
    platforms="Posix; MacOS X; Windows",
    test_suite='nose.collector',
    entry_points='''
        [console_scripts]
        course_parser=course_parser.scripts.cli:main
    ''',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Topic :: Software Development :: Testing',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
)



"""
    parser for saint

    __        __               _               _   _
    \ \      / /_ _ _ __ _ __ (_)_ __   __ _  | | | |
     \ \ /\ / / _` | '__| '_ \| | '_ \ / _` | | | | |
      \ V  V / (_| | |  | | | | | | | | (_| | |_| |_|
       \_/\_/ \__,_|_|  |_| |_|_|_| |_|\__, | (_) (_)
                                       |___/


     _   _                      ____
    | \ | | ___   ___  _ __    / ___| __ _ _ __   __ _
    |  \| |/ _ \ / _ \| '_ \  | |  _ / _` | '_ \ / _` |
    | |\  | (_) | (_) | | | | | |_| | (_| | | | | (_| |
    |_| \_|\___/ \___/|_| |_|  \____|\__,_|_| |_|\__, |
                                                 |___/

    눈갱 주의
"""

from bs4 import element
import re
import ast


def get_tab_item_index(base_soup, section):
    """
    ItemIndex 를 얻는다.
    :param elem:
    Saint.soup_jar['base']
    :param section:
    '학부전공별', '교양필수', '교양선택', ...
    :return:
    """
    elem = base_soup.find('span', text=section).parent.parent
    lsdata = elem.get('lsdata')
    lsdata = lsdata.replace('true', 'True')
    lsdata_dict = ast.literal_eval(lsdata)
    return lsdata_dict[1]


def get_skey(elem):
    if type(elem) == element.Tag:
        skey_yaml = elem.get('lsdata')
        return ast.literal_eval(skey_yaml)[0]
    else:
        raise Exception("Unexpected elem type {}".format(type(elem)))


def get_liberal_arts_key(soup_grade):
    """
    :param soup_grade:
    Saint.soup_jar['grade']
    :return:
    'WD01C2'
    """
    label = soup_grade.find('label', text='과목명')
    return label.get('f')


def get_selective_id(soup_selective):
    """
    교양선택 탭에서 분야의 Id를 얻는다
    :param soup_selective:
    Saint.soup_jar['교양선택']
    :return:
    """
    label = soup_selective.find('label', text='분야')
    return label.get('f')


def get_year_key(soup_base):
    label = soup_base.find('label', text='학년도')
    year_key = label.get('f')
    return year_key


def get_semester_key(soup_base):
    label = soup_base.find('label', text='학기')
    semester_key = label.get('f')
    return semester_key


def get_sap_wd_secure_id(soup_base):
    form = soup_base.find('form', {'id': 'sap.client.SsrClient.form'})
    secure_id = form.find('input', {'id': 'sap-wd-secure-id'})
    sap_wd_secure_id = secure_id.get('value')
    return sap_wd_secure_id


def get_selective_course_skey(selective_soup, course_name):
    course_td = selective_soup.find('td', text=course_name)
    course_tr = course_td.parent
    course_yaml = course_tr.get('lsdata')
    course_skey = ast.literal_eval(course_yaml)[0]
    return course_skey


def get_semester_skey(soup_base, semester):
    if semester in ['1 학기', '여름학기', '2 학기', '겨울학기']:
        semester = soup_base.find('tr', text=semester)
        skey_yaml = semester.get('lsdata')
        semester_skey = ast.literal_eval(skey_yaml)[0]
        return semester_skey
    else:
        raise Exception(
            "parameter semester is expected in ['1 학기', '여름학기', '2 학기', '겨울학기'] but {} token".format(semester))


def get_faculty_key(soup_base):
    faculty = soup_base.find('input', {"lsdata": re.compile(r"{1:'2.2em',8:"),
                                     "lsevents": "{Select:[{ResponseData:'delta',ClientAction:'submit'},{}]}"})
    faculty_key = faculty.get('id')
    return faculty_key


def get_college_key(soup_base):
    col_input = soup_base.find('input', {"lsdata": re.compile(r"{1:'14.3em',")})  # 단과대학
    college_key = col_input.get('id')
    return college_key


def get_major_key(soup_base):
    major = soup_base.find('input', {"lsdata": re.compile(r"{1:'2.2em',8:"),
                                      "lsevents": "{Select:[{ResponseData:'delta',EnqueueCardinality:'single'},{}]}"})
    major_key = major.get('id')
    return major_key


def get_liberal_arts_skey(soup_grade, course_name):
    """
    :param soup_grade:
    Saint.soup_jar['grade']
    :param course_name:
    '컴퓨팅적사고'
    :return:
    """
    course_elem = soup_grade.find('td', text=course_name).parent
    course_skey = get_skey(course_elem)
    return course_skey


def get_major_skey(soup_major, major):
    major_skey_elem = soup_major.find_all('tr', text=major)
    major_skey_elem = major_skey_elem[len(major_skey_elem) - 1]
    major_skey = get_skey(major_skey_elem)
    return major_skey


def get_search_id(soup_base):
    search_elem = soup_base.find("div", {"lsdata": re.compile(r"{0:'검색',4:'찾기',10:")})
    search_id = search_elem.get('id')
    return search_id


def get_college_skey(soup_base, college):
    """
    :param soup_base:
    Saint.soup_jar['semester']
    :param college:
    :return:
    """
    col_elem = soup_base.find('tr', text=college)  # 인문대학 법과대학 등등....
    col_skey = get_skey(col_elem)
    return col_skey


def get_faculty_skey(soup_faculty, faculty):
    faculty_skey_elem = soup_faculty.find('tr', text=faculty)
    faculty_skey = get_skey(faculty_skey_elem)
    return faculty_skey


def parse_subjects(search_soup):
    """
    :param search_soup:
    SoupParser.soup_jar['search']
    :return:
    list
    subjects
    """
    subjects = []
    th = search_soup.select('th')
    tr = search_soup.find_all('tr', {'role': 'row'})

    keys = [key.text for key in th]
    for row in tr:
        cells = row.find_all('td')
        if len(cells) == 1:
            return subjects
        subject = dict((keys[i], cells[i].get_text("\n")) for i in range(15))
        subjects.append(subject)
    return subjects


def get_liberal_arts_courses(grade_soup):
    """
    :param grade_soup:
    SoupParser.soup_jar['grade']
    :return:
    """
    content = grade_soup.find('content')
    divs = content.find_all('div', {'class': 'lsItemlistbox__root'})
    if len(divs) != 2:
        raise Exception()

    course_divs = divs[1]
    tds = course_divs.find_all('td')
    liberal_arts_courses = []
    for td in tds:
        liberal_arts_courses.append(td.text)
    return liberal_arts_courses


def get_colleges(semester_soup):
    """
    college를 아무거나 선택해서 soup_jar['college']를 확보한후 실행한다.
    :param semester_soup:
    SoupParser.soup_jar['college']
    :return:
    [
    '인문대학',
    '자연과학대학',
    '법과대학',
    '사회과학대학',
    '경제통상대학',
    '경영대학',
    '공과대학',
    'IT대학',
    '베어드학부대학',
    '예술창작학부',
    '스포츠학부',
    '융합특성화자유전공학부']
    """
    content = semester_soup.find('content')
    college_div = content.find('div')
    tds = college_div.tbody.find_all('td')
    colleges = []
    for td in tds:
        colleges.append(td.text)
    return colleges


def get_faculties(college_soup):
    """
    :param college_soup:
     SoupParser.soup_jar['college']
    :return:
    ['전자정보공학부 전자공학전공',
    '전자정보공학부 IT융합전공',
    '컴퓨터학부',
    '소프트웨어학부',
    '정보통신전자공학부',
    '스마트시스템소프트웨어학과',
    '글로벌미디어학부',
    '미디어경영학과']
    """
    content = college_soup.find('content')
    first_div = content.find('div')
    siblings = first_div.find_next_siblings()
    if len(siblings) == 2:
        major_div = first_div.find_next_siblings()[0]
        tds = major_div.tbody.find_all('td')
        faculties = []
        for td in tds:
            faculties.append(td.text)
        return faculties
    else:
        raise Exception()


def get_majors(faculty_soup):
    """
    :param faculty_soup:
    SoupParser.soup_jar['faculty']
    :return:
    """
    content = faculty_soup.find('content')
    first_div = content.find('div')
    siblings = first_div.find_next_siblings()
    if len(siblings) == 2:
        major_div = first_div.find_next_siblings()[1]
        tds = major_div.tbody.find_all('td')
        majors = []
        for td in tds:
            majors.append(td.text)
            if '' in majors:
                majors.remove('')
        return majors
    else:
        raise Exception()


def get_selective_courses(selective_soup):
    """
    :param selective_soup:
    self.soup_jar['교양선택']
    :return:
    """
    course_td = selective_soup.find('td', text='전체')
    course_tr = course_td.parent
    nexts = course_tr.find_next_siblings()
    prevs = course_tr.find_previous_siblings()
    courses = ['전체']
    for n in nexts:
        courses.append(n.text.strip())
    for p in prevs:
        courses.append(p.text.strip())

    return courses


def get_section_id(base_soup):
    """
    학부전공별, 교양필수, 교양선택 등 항목을 선택하는데 필요한 id를 얻는다
    :param base_soup:
    SoupParser.soup_jar['base']
    :return:
    """
    table = base_soup.find('table', {
        'lsevents': "{TabSelect:[{ResponseData:'delta',ClientAction:'submit'},{}]," +
                    "Scroll:[{ResponseData:'delta',EnqueueCardinality:'single'},{}]," +
                    "Hotkey:[{ResponseData:'delta',ClientAction:'submit'},{}]}"
    })
    return table.get('id')


def get_tab_id(base_soup, tab_name):
    """
    학부전공별, 교양필수, 교양선택등 이 있는 탭의 id를 얻는다
    :param base_soup:
    SoupParser.soup_jar['base']
    :param tab_name:
    '학부전공별', '교양필수', '교양선택', ...
    :return:
    """
    tab_span = base_soup.find('span', text=tab_name)
    return tab_span.parent.parent.get('id')


def get_grade_id_from_liberal_arts_tab(liberal_arts_soup):
    """
    :param liberal_arts_soup:
    SoupParser.soup_jar['교양선택']
    :param grade:
    '전체학년', '1학년', '2학년', '3학년', '4학년', '5학년'
    :return:
    """
    # tr = liberal_arts_soup.find('tr', text=grade)
    # return tr.get('id')
    return liberal_arts_soup.find('span', text='과목 수준').label.get('f')


def get_grade_skey_from_liberal_arts_tab(liberal_arts_soup, grade):
    """
    :param liberal_arts_soup:
    SoupParser.soup_jar['교양선택']
    :param grade:
    '전체학년', '1학년', '2학년', '3학년', '4학년', '5학년'
    :return:
    """
    tr = liberal_arts_soup.find('tr', text=grade)
    lsdata = tr.get('lsdata')
    lsdata_dict = ast.literal_eval(lsdata)
    return lsdata_dict[0]




# -*- coding: utf-8 -*-

"""
This module provides functions for searching the courses
in command line interface by pysaint

"""
from __future__ import absolute_import

import click

from pysaint.saint import Saint


@click.group()
def main():
    """
    TODO: Make CLI Interface
    :return:
    """
    pass



import os
import errno
import pickle
from json import load
import json


def save_pickle(directory, file_name, python_object):
    if not os.path.exists(directory):
        try:
            os.makedirs(os.path.dirname(directory))
        except OSError as exc:
            if exc.errno != errno.EEXIST:
                raise

    if directory[-1] != '/':
        directory += '/'
    full_path = directory + file_name

    with open(full_path, "wb") as _f:
        pickle.dump(python_object, _f, protocol=pickle.HIGHEST_PROTOCOL)


def write_file(directory, file_name, content):
    if directory[-1] != '/':
        directory += '/'
    full_path = directory + file_name

    f = open(full_path, 'w')
    f.write(content)
    f.close()


def load_pickle(directory, file_name):
    if directory[-1] != '/':
        directory += '/'
    full_path = directory + file_name

    if not os.path.exists(full_path):
        raise Exception('cannot find {}'.format(full_path))

    with open(full_path, "rb") as _f:
        return pickle.load(_f)


def load_map(path='./map.json'):
    return load(open(path, 'rb'))


def dictionary_to_json(py_dict):
    """
    :param py_dict:
    python dictionary
    :return:
    serialized json string
    """
    return json.dumps(py_dict, ensure_ascii=False, indent=4)


def save_json(directory, file_name, py_dict):
    if not os.path.exists(directory):
        try:
            os.makedirs(os.path.dirname(directory))
        except OSError as exc:
            if exc.errno != errno.EEXIST:
                raise

    if directory[-1] != '/':
        directory += '/'

    json_string = dictionary_to_json(py_dict)
    write_file(directory, file_name, json_string)



from .constants import SOURCE_URL, ECC_URL, REQUEST_HEADERS, SESSION_HEADERS
from .parser import *
from . import sap_event_queue
import requests
from bs4 import BeautifulSoup
from collections import defaultdict


class Saint:
    def __init__(self):
        self.sess = requests.Session()
        self.soup_jar = defaultdict()
        self.sess.headers = SESSION_HEADERS
        self.sess.cookies.update({'GWMESSENGER': 'stopMessenger'})

        res_init = self.sess.get(SOURCE_URL)
        self.soup_jar['init'] = BeautifulSoup(res_init.text, 'html.parser')

        form = self.soup_jar['init'].find('form', {'name': 'sap.client.SsrClient.form'})
        self.action = form.get('action')
        res_base = self.sess.post(ECC_URL + self.action)

        self.soup_jar['base'] = BeautifulSoup(res_base.text, 'lxml')

        self.sap_wd_secure_id = get_sap_wd_secure_id(self.soup_jar['base'])
        self.year_key = get_year_key(self.soup_jar['base'])
        self.semester_key = get_semester_key(self.soup_jar['base'])
        self.college_key = get_college_key(self.soup_jar['base'])
        self.faculty_key = get_faculty_key(self.soup_jar['base'])
        self.major_key = get_major_key(self.soup_jar['base'])
        self.search_id = get_search_id(self.soup_jar['base'])

    def select_year(self, year):
        """
        :param year:
        String
        '2016' '2017' 등등.... 정규식으로 4자리 숫자인지 확인할 필요가 있음
        :return:
        """
        dt_year = sap_event_queue.combo_select(self.year_key, year, self.sap_wd_secure_id)
        pg_year = self.sess.post(ECC_URL + self.action, data=dt_year)

    def select_semester(self, semester):
        """
        학기를 선택한다.
        :param semester:
        String
        '1 학기'
        :return:
        """
        semester_skey = get_semester_skey(self.soup_jar['base'], semester)
        dt_semester = sap_event_queue.combo_select(self.semester_key, semester_skey, self.sap_wd_secure_id)

        after_semester_click = self.sess.post(ECC_URL + self.action, data=dt_semester)
        self.soup_jar['semester'] = BeautifulSoup(after_semester_click.text, 'lxml')

    def select_course_section(self, section):
        """
        학부전공별, 교양필수, 교양선택등 이 있는 탭을 선택한다
        :param section:
        String
        '교양필수', '교양선택',
        :return:
        """
        section_id = get_section_id(self.soup_jar['base'])
        tab_id = get_tab_id(self.soup_jar['base'], section)
        item_index = get_tab_item_index(self.soup_jar['base'], section)
        dt_tab = sap_event_queue.tab_select(section_id, tab_id, item_index, self.sap_wd_secure_id)

        after_course_section_click = self.sess.post(
            ECC_URL + self.action, data=dt_tab)
        self.soup_jar['{}'.format(section)] = BeautifulSoup(after_course_section_click.text, 'lxml')

    def _select_college(self, college):
        """
        단과대학을 선택한다.
        :param college:
        String
        인문대학, IT대학, ..
        :return:
        """
        college_skey = get_college_skey(self.soup_jar['semester'], college)
        dt_college = sap_event_queue.combo_select(self.college_key, college_skey, self.sap_wd_secure_id)

        after_college_click = self.sess.post(ECC_URL + self.action, data=dt_college)

        self.soup_jar['college'] = BeautifulSoup(after_college_click.text, 'lxml')

    def _select_faculty(self, faculty):
        """
        학부를 선택한다.
        :param faculty:
        String
        철학과, 정보통신전자공학부, ...
        :return:
        """
        faculty_skey = get_faculty_skey(self.soup_jar['college'], faculty)
        dt_faculty = sap_event_queue.combo_select(self.faculty_key, faculty_skey, self.sap_wd_secure_id)

        after_faculty_click = self.sess.post(ECC_URL + self.action, data=dt_faculty)
        self.soup_jar['faculty'] = BeautifulSoup(after_faculty_click.text, 'lxml')

    def _select_major(self, major):
        """
        학과 선택과 동시에 search_button을 눌러서 subjects 정보를 받아온다
        :param major:
        String
        '정보통신전자공학부'
        :return:
        list
        subjects
        """
        major_skey = get_major_skey(self.soup_jar['faculty'], major)
        dt_major = sap_event_queue.combo_select_with_button_press(self.major_key, major_skey, self.search_id, self.sap_wd_secure_id)
        after_search_click = self.sess.post(ECC_URL + self.action, data=dt_major)

        self.soup_jar['search'] = BeautifulSoup(after_search_click.text, 'lxml')
        subjects = parse_subjects(self.soup_jar['search'])
        return subjects

    def _select_uni_faculty_major(self, major):
        """
        학부가 1개밖에 없는 학과를 선택함
        soup_jar['faculty']가 없어서 따로 처리해줌
        :param major:
        스포츠학부
        :return:
        """
        major_skey = get_major_skey(self.soup_jar['college'], major)
        dt_major = sap_event_queue.combo_select_with_button_press(self.major_key, major_skey, self.search_id,
                                                                  self.sap_wd_secure_id)
        after_search_click = self.sess.post(ECC_URL + self.action, data=dt_major)

        self.soup_jar['search'] = BeautifulSoup(after_search_click.text, 'lxml')
        subjects = parse_subjects(self.soup_jar['search'])
        return subjects

    def _select_search_button(self):
        """
        학부에 학과가 1개일 경우 학과를 선택하지 않고 이 메소드를 호출한다.
        바로 search press event 를 전송해서 정보를 받아 올 수 있다.
        :return:
        list
        subjects
        """
        dt_click_search = sap_event_queue.button_press(self.search_id, self.sap_wd_secure_id)
        after_search_click = self.sess.post(ECC_URL + self.action, data=dt_click_search)

        self.soup_jar['search'] = BeautifulSoup(after_search_click.text, 'lxml')
        subjects = parse_subjects(self.soup_jar['search'])
        return subjects

    def select_on_major(self, college, faculty, major):
        """
        :param college: 단과대학
        :param faculty: 학부
        :param major: 학과
        :return:
        list
        subjects
        """
        if major == '':
            return None
        self._select_college(college)
        faculties = get_faculties(self.soup_jar['college'])
        if len(faculties) == 1:
            # 스포츠학부 같은 경우 단과대학 선택후 학부가 1개라서 바로 학부가 선택됨
            majors = get_majors(self.soup_jar['college'])
            if major in majors:
                return self._select_uni_faculty_major(major)
            else:
                print("major: {} is not in majors: {}".format(major, majors))
                return None
        else:
            self._select_faculty(faculty)
            majors = get_majors(self.soup_jar['faculty'])
        if len(majors) == 1:
            return self._select_search_button()
        return self._select_major(major)

    def _select_grade(self, grade):
        """
        교양필수 탭에서 학년을 선택한다.
        선행작업:
        CourseParser.select_year()
        CourseParser.select_semester()
        CourseParser.select_course_section()
        :param grade:
        '전체학년', '1학년', '2학년', '3학년', '4학년', '5학년'
        :return:
        """
        grade_id = get_grade_id_from_liberal_arts_tab(self.soup_jar['교양필수'])
        grade_skey = get_grade_skey_from_liberal_arts_tab(self.soup_jar['교양필수'], grade)
        dt_grade = sap_event_queue.combo_select(grade_id, grade_skey, self.sap_wd_secure_id)

        after_select_grade = self.sess.post(
            ECC_URL + self.action, data=dt_grade
        )
        grade_soup = BeautifulSoup(after_select_grade.text, 'lxml')
        self.soup_jar['grade'] = grade_soup
        return grade_soup

    def _select_selective_liberal_course(self, course_name):
        """
        교양선택에서 과목을 선택하고 검색 버튼을 누른다.
        선행작업:
        CourseParser.select_course_section('교양선택')
        :param course_name:
        '전체', '*문학과 예술',
        :return:
        """
        course_id = get_selective_id(self.soup_jar['교양선택'])
        course_skey = get_selective_course_skey(self.soup_jar['교양선택'], course_name)
        search_id = get_search_id(self.soup_jar['교양선택'])
        dt_combo_with_search_button_press \
            = sap_event_queue.combo_select_with_button_press(course_id, course_skey, search_id, self.sap_wd_secure_id)
        on_selective_liberal_course = self.sess.post(ECC_URL + self.action, data=dt_combo_with_search_button_press)
        self.soup_jar['selective_search'] = BeautifulSoup(on_selective_liberal_course.text, 'lxml')
        return parse_subjects(self.soup_jar['selective_search'])

    def _select_liberal_arts(self, course_name):
        """
        선행작업:
        CoursePraser._select_grade()
        교양필수 과목명을 선택하고 검색버튼을 누른다.
        :param course_name:
        '컴퓨팅적사고'
        :return:
        """
        search_id = get_search_id(self.soup_jar['grade'])
        course_key = get_liberal_arts_key(self.soup_jar['grade'])
        course_skey = get_liberal_arts_skey(self.soup_jar['grade'], course_name)
        dt_combo_with_search_button_press \
            = sap_event_queue.combo_select_with_button_press(course_key, course_skey, search_id, self.sap_wd_secure_id)
        on_liberal_arts_search_click = self.sess.post(ECC_URL + self.action, data=dt_combo_with_search_button_press)
        self.soup_jar['liberal_search'] = BeautifulSoup(on_liberal_arts_search_click.text, 'lxml')
        return parse_subjects(self.soup_jar['liberal_search'])

    def select_on_liberal_arts(self, grade, course_name):
        """
        :param grade:
        :param course_name:
        :return:
        subject lists
        """
        self._select_grade(grade)
        courses = get_liberal_arts_courses(self.soup_jar['grade'])
        if course_name not in courses:
            raise Exception()
        return self._select_liberal_arts(course_name)

    def select_on_selective_liberal(self, course_name):
        """
        교양선택에서 과목을 선택해서 검색 정보를 얻는다
        선행작업:
        CourseParser.select_course_section('교양선택')
        :param course_name:
        '전체', '*문학과 예술(융합-인문)', ...
        :return:
        """
        return self._select_selective_liberal_course(course_name)

    def get_liberal_arts_map(self):
        """
        학년, 교양필수_과목 맵을 얻는다.
        선행되야하는 작업:
        CourseParser.select_course_section('교양필수')
        CourseParser.select_year()
        CourseParser.select_semester()
        :return:
        {
            '전체학년': ['CHAPEL'],
            '1학년': ['숭실인의인성과소양', '현대인과성서', '현대인과성서세미나', ...],
            ...
        }
        """
        grades = ['전체학년', '1학년', '2학년', '3학년', '4학년', '5학년']
        liberal_arts_map = {grade: {} for grade in grades}
        self.select_course_section('교양필수')
        for grade in grades:
            self._select_grade(grade)
            majors = get_liberal_arts_courses(self.soup_jar['grade'])
            liberal_arts_map[grade] = majors
        return liberal_arts_map

    def get_major_map(self):
        """
        단과대 학부 전공 맵을 얻는다.
        선행되야하는 작업:
        CourseParser.select_year()
        CourseParser.select_semester()
        :return:
        {
            '인문대학': {
                '철학과': ['철학과'],
                ...
            },
            'IT대학': {
                '정보통신전자공학부': ['정보통신전자공학부', '전자공학과'],
                ...
            },
            ...
        }
        """
        self._select_college('인문대학')
        colleges = get_colleges(self.soup_jar['college'])
        selection_map = {key: {} for key in colleges}
        for college in colleges:
            self._select_college(college)
            faculties = get_faculties(self.soup_jar['college'])
            faculties_map = {key: {} for key in faculties}
            selection_map[college] = faculties_map
            for faculty in faculties:
                if len(faculties) == 1:
                    majors = get_majors(self.soup_jar['college'])
                else:
                    self._select_faculty(faculty)
                    majors = get_majors(self.soup_jar['faculty'])
                selection_map[college][faculty] = majors
        return selection_map

    def get_selective_liberal_map(self):
        """
        :TODO :
        CourseParser.select_course_section('교양선택') 을 처음 실행해서 얻게된
        self.soup_jar['교양선택'] 에 과목정보가 있는경우가 있고 없는경우가 있다.
        과목정보가 변경되면 과목정보가 생기고 그대로면 안생기는듯
        교양선택 맵을 얻는다.
        선행되야하는 작업:
        :return:
        """
        self.select_course_section('교양선택')
        courses = get_selective_courses(self.soup_jar['교양선택'])
        return courses


if __name__ == '__main__':
    course_parser = Saint()
    course_parser.select_year('2017')
    course_parser.select_semester('1 학기')
    results = course_parser.select_on_major('인문대학', '철학과', '철학과')

    print(results)

